@@ -1,7 +1,32 @@
 import numpy as np
+from astropy.wcs import (WCSSUB_SPECTRAL, WCSSUB_LONGITUDE, WCSSUB_LATITUDE)
 from . import wcs_utils
-import warnings
+from astropy import log
 
+def _fix_spectral(wcs):
+    """
+    Attempt to fix a cube with an invalid spectral axis definition.  Only uses
+    well-known exceptions, e.g. CTYPE = 'VELOCITY'.  For the rest, it will try
+    to raise a helpful error.
+    """
+
+    axtypes = wcs.get_axis_types()
+
+    types = [a['coordinate_type'] for a in axtypes]
+
+    if wcs.naxis not in (3,4):
+        raise TypeError("The WCS has {0} axes of types {1}".format(len(types),
+                                                                   types))
+
+    # sanitize noncompliant headers
+    if 'spectral' not in types:
+        log.warn("No spectral axis found; header may be non-compliant.")
+        for ind,tp in enumerate(types):
+            if tp not in ('celestial','stokes'):
+                if wcs.wcs.ctype[ind] in wcs_utils.bad_spectypes_mapping:
+                    wcs.wcs.ctype[ind] = wcs_utils.bad_spectypes_mapping[wcs.wcs.ctype[ind]]
+
+    return wcs
 
 def _split_stokes(array, wcs):
     """
@@ -18,32 +43,61 @@ def _split_stokes(array, wcs):
         dimension, and a Stokes dimension.
     """
 
-    if array.ndim != 4:
-        raise ValueError("Input array must be 4-dimensional")
+    if array.ndim not in (3,4):
+        raise ValueError("Input array must be 3- or 4-dimensional for a"
+                         " STOKES cube")
 
     if wcs.wcs.naxis != 4:
-        raise ValueError("Input WCS must be 4-dimensional")
+        raise ValueError("Input WCS must be 4-dimensional for a STOKES cube")
+
+    wcs = _fix_spectral(wcs)
 
     # reverse from wcs -> numpy convention
     axtypes = wcs.get_axis_types()[::-1]
 
     types = [a['coordinate_type'] for a in axtypes]
 
-    # Find stokes dimension
-    stokes_index = types.index('stokes')
+    try:
+        # Find stokes dimension
+        stokes_index = types.index('stokes')
+    except ValueError:
+        # stokes not in list, but we are 4d
+        if types.count('celestial') == 2 and types.count('spectral') == 1:
+            if None in types:
+                stokes_index = types.index(None)
+                log.warn("FITS file has no STOKES axis, but it has a blank"
+                         " axis type at index {0} that is assumed to be "
+                         "stokes.".format(4-stokes_index))
+            else:
+                for ii,tp in enumerate(types):
+                    if tp not in ('celestial', 'spectral'):
+                        stokes_index = ii
+                        stokes_type = tp
+
+                log.warn("FITS file has no STOKES axis, but it has an axis"
+                         " of type {1} at index {0} that is assumed to be "
+                         "stokes.".format(4-stokes_index, stokes_type))
+        else:
+            raise IOError("There are 4 axes in the data cube but no STOKES "
+                          "axis could be identified")
 
     # TODO: make the stokes names more general
     stokes_names = ["I", "Q", "U", "V"]
 
     stokes_arrays = {}
 
-    wcs_slice = wcs_utils.drop_axis(wcs, array.ndim - 1 - stokes_index)
+    wcs_slice = wcs_utils.drop_axis(wcs, wcs.naxis - 1 - stokes_index)
 
-    for i_stokes in range(array.shape[stokes_index]):
+    if array.ndim == 4:
+        for i_stokes in range(array.shape[stokes_index]):
 
-        array_slice = [i_stokes if idim == stokes_index else slice(None) for idim in range(array.ndim)]
+            array_slice = [i_stokes if idim == stokes_index else slice(None)
+                           for idim in range(array.ndim)]
 
-        stokes_arrays[stokes_names[i_stokes]] = array[array_slice]
+            stokes_arrays[stokes_names[i_stokes]] = array[array_slice]
+    else:
+        # 3D array with STOKES as a 4th header parameter
+        stokes_arrays['I'] = array
 
     return stokes_arrays, wcs_slice
 
@@ -70,19 +124,13 @@ def _orient(array, wcs):
     if wcs.wcs.naxis != 3:
         raise ValueError("Input WCS must be 3-dimensional")
 
+    wcs = _fix_spectral(wcs)
+
     # reverse from wcs -> numpy convention
     axtypes = wcs.get_axis_types()[::-1]
 
     types = [a['coordinate_type'] for a in axtypes]
 
-    # sanitize noncompliant headers
-    if 'spectral' not in types:
-        warnings.warn("No spectral axis found; header may be non-compliant.")
-        types = [tp if tp == 'celestial' else 'spectral' for tp in types]
-        spec = types.index('spectral')
-        if wcs.wcs.ctype[spec] in wcs_utils.bad_spectypes_mapping:
-            wcs.wcs.ctype[spec] = wcs_utils.bad_spectypes_mapping[wcs.wcs.ctype[spec]]
-
     nums = [None if a['coordinate_type'] != 'celestial' else a['number']
             for a in axtypes]
 
@@ -92,8 +140,7 @@ def _orient(array, wcs):
     t = [types.index('spectral'), nums.index(1), nums.index(0)]
     result_array = array.transpose(t)
 
-    t = wcs.wcs.naxis - np.array(t[::-1]) - 1
-    result_wcs = wcs_utils.reindex_wcs(wcs, t)
+    result_wcs = wcs.sub([WCSSUB_LONGITUDE, WCSSUB_LATITUDE, WCSSUB_SPECTRAL])
 
     return result_array, result_wcs
 
@@ -144,6 +191,12 @@ class SliceIndexer(object):
         raise Exception("You need to specify a slice (e.g. ``[:]`` or "
                         "``[0,:,:]`` in order to access this property.")
 
+def is_huge(cube, threshold=1e8):
+    if cube.size < threshold:  # smallish
+        return False
+    else:
+        return True
+
 
 def iterator_strategy(cube, axis=None):
     """
@@ -167,6 +220,6 @@ def iterator_strategy(cube, axis=None):
         *ray*  recommends working with one ray at a time
     """
     # pretty simple for now
-    if np.product(cube.shape) < 1e8:  # smallish
+    if cube.size < 1e8:  # smallish
         return 'cube'
     return 'slice'
