@@ -4,9 +4,11 @@ import numpy as np
 from numpy.lib.stride_tricks import as_strided
 
 from . import wcs_utils
+from .lower_dimensional_structures import Projection
+
 
 __all__ = ['InvertedMask', 'CompositeMask', 'BooleanArrayMask',
-           'LazyMask', 'FunctionMask']
+           'LazyMask', 'LazyComparisonMask', 'FunctionMask']
 
 # Global version of the with_spectral_unit docs to avoid duplicating them
 with_spectral_unit_docs = """
@@ -25,14 +27,63 @@ with_spectral_unit_docs = """
             wavelength/frequency can be overridden with this parameter.
         """
 
-def is_broadcastable(shp1, shp2):
+def is_broadcastable_and_smaller(shp1, shp2):
+    """
+    Test if shape 1 can be broadcast to shape 2, not allowing the case
+    where shape 2 has a dimension length 1
+    """
     for a, b in zip(shp1[::-1], shp2[::-1]):
-        if a == 1 or b == 1 or a == b:
+        # b==1 is broadcastable but not desired
+        if a == 1 or a == b:
             pass
         else:
             return False
     return True
 
+def dims_to_skip(shp1, shp2):
+    """
+    For a shape `shp1` that is broadcastable to shape `shp2`, specify which
+    dimensions are length 1.
+
+    Parameters
+    ----------
+    keep : bool
+        If True, return the dimensions to keep rather than those to remove
+    """
+    if not is_broadcastable_and_smaller(shp1, shp2):
+        raise ValueError("Cannot broadcast {0} to {1}".format(shp1,shp2))
+    dims = []
+
+    for ii,(a, b) in enumerate(zip(shp1[::-1], shp2[::-1])):
+        # b==1 is broadcastable but not desired
+        if a == 1:
+            dims.append(len(shp2) - ii - 1)
+        elif  a == b:
+            pass
+        else:
+            raise ValueError("This should not be possible")
+
+    if len(shp1) < len(shp2):
+        dims += list(range(len(shp2)-len(shp1)))
+
+    return dims
+
+def view_of_subset(shp1, shp2, view):
+    """
+    Given two shapes and a view, assuming that shape 1 can be broadcast
+    to shape 2, return the sub-view that applies to shape 1
+    """
+    dts = dims_to_skip(shp1, shp2)
+    if view:
+        cv_view = [x for ii,x in enumerate(view) if ii not in dts]
+    else:
+        # if no view is specified, still need to slice
+        cv_view = [x for ii,x in enumerate([slice(None)]*3)
+                   if ii not in dts]
+
+    return cv_view
+
+
 class MaskBase(object):
 
     __metaclass__ = abc.ABCMeta
@@ -119,7 +170,10 @@ class MaskBase(object):
         This is an internal method used by :class:`SpectralCube`.
         Users should use the property :meth:`MaskBase.filled_data`
         """
-        sliced_data = data[view].copy().astype(np.float)
+        # Must convert to floating point, but should not change from inherited
+        # type otherwise
+        dt = np.find_common_type([data.dtype], [np.float])
+        sliced_data = data[view].astype(dt)
         ex = self.exclude(data=data, wcs=wcs, view=view)
         sliced_data[ex] = fill
         return sliced_data
@@ -134,7 +188,28 @@ class MaskBase(object):
         return InvertedMask(self)
 
     def __getitem__(self):
-        raise NotImplementedError("Slicing not supported by mask class {0}".format(self.__class__.__name__))
+        raise NotImplementedError("Slicing not supported by mask class {0}"
+                                  .format(self.__class__.__name__))
+
+    def quicklook(self, view, wcs=None, filename=None, use_aplpy=True):
+        '''
+        View a 2D slice of the mask, specified by view.
+
+        Parameters
+        ----------
+        view : tuple
+            Slicing to apply to the mask. Must return a 2D slice.
+        wcs : astropy.wcs.WCS, optional
+            WCS object to use in plotting the mask slice.
+        filename : str, optional
+            Filename of the output image. Enables saving of the plot.
+        '''
+
+        view_twod = self.include(view=view)
+
+        proj = Projection(view_twod, wcs=wcs)
+
+        proj.quicklook(filename=filename, use_aplpy=use_aplpy)
 
     def _get_new_wcs(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -148,6 +223,7 @@ class MaskBase(object):
 
         newwcs = convert_spectral_axis(self._wcs, unit, out_ctype,
                                        rest_value=rest_value)
+        newwcs.wcs.set()
 
         return newwcs
 
@@ -164,7 +240,7 @@ class InvertedMask(MaskBase):
 
     def __getitem__(self, view):
         return InvertedMask(self._mask[view])
-    
+
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
         Get an InvertedMask copy with a WCS in the modified unit
@@ -210,7 +286,8 @@ class CompositeMask(MaskBase):
             raise ValueError("Operation '{0}' not supported".format(self._operation))
 
     def __getitem__(self, view):
-        return CompositeMask(self._mask1[view], self._mask2[view], operation=self._operation)
+        return CompositeMask(self._mask1[view], self._mask2[view],
+                             operation=self._operation)
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -252,7 +329,7 @@ class BooleanArrayMask(MaskBase):
         #if mask.ndim != 3 and (shape is None or len(shape) != 3):
         #    raise ValueError("When creating a BooleanArrayMask with <3 dimensions, "
         #                     "the shape of the 3D array must be specified.")
-        if shape is not None and not is_broadcastable(mask.shape, shape):
+        if shape is not None and not is_broadcastable_and_smaller(mask.shape, shape):
             raise ValueError("Mask cannot be broadcast to the specified shape.")
         self._shape = shape or mask.shape
         n_extra_dims = (len(self._shape)-mask.ndim)
@@ -263,14 +340,17 @@ class BooleanArrayMask(MaskBase):
         else:
             self._mask = mask
 
-    def _validate_wcs(self, new_data, new_wcs):
-        if not is_broadcastable(new_data.shape, self._mask.shape):
+    def _validate_wcs(self, new_data=None, new_wcs=None):
+        if new_data is not None and not is_broadcastable_and_smaller(self._mask.shape,
+                                                                     new_data.shape):
             raise ValueError("data shape cannot be broadcast to match mask shape")
-        if new_wcs not in self._wcs_whitelist:
-            if not wcs_utils.check_equality(new_wcs, self._wcs,
-                                            warn_missing=True):
-                raise ValueError("WCS does not match mask WCS")
-        self._wcs_whitelist.add(new_wcs)
+        if new_wcs is not None:
+            if new_wcs not in self._wcs_whitelist:
+                if not wcs_utils.check_equality(new_wcs, self._wcs,
+                                                warn_missing=True):
+                    raise ValueError("WCS does not match mask WCS")
+                else:
+                    self._wcs_whitelist.add(new_wcs)
 
     def _include(self, data=None, wcs=None, view=()):
         result_mask = self._mask[view]
@@ -340,21 +420,25 @@ class LazyMask(MaskBase):
 
         self._wcs_whitelist = set()
 
-    def _validate_wcs(self, new_data, new_wcs):
-        if not is_broadcastable(new_data.shape, self._data.shape):
-            raise ValueError("data shape cannot be broadcast to match mask shape")
-        if new_wcs not in self._wcs_whitelist:
-            if not wcs_utils.check_equality(new_wcs, self._wcs,
-                                            warn_missing=True):
-                raise ValueError("WCS does not match mask WCS")
-        self._wcs_whitelist.add(new_wcs)
+    def _validate_wcs(self, new_data=None, new_wcs=None):
+        if new_data is not None:
+            if not is_broadcastable_and_smaller(new_data.shape, self._data.shape):
+                raise ValueError("data shape cannot be broadcast to match mask shape")
+        if new_wcs is not None:
+            if new_wcs not in self._wcs_whitelist:
+                if not wcs_utils.check_equality(new_wcs, self._wcs,
+                                                warn_missing=True):
+                    raise ValueError("WCS does not match mask WCS")
+                else:
+                    self._wcs_whitelist.add(new_wcs)
 
     def _include(self, data=None, wcs=None, view=()):
         self._validate_wcs(data, wcs)
         return self._function(self._data[view])
 
     def __getitem__(self, view):
-        return LazyMask(self._function, data=self._data[view], wcs=wcs_utils.slice_wcs(self._wcs, view))
+        return LazyMask(self._function, data=self._data[view],
+                        wcs=wcs_utils.slice_wcs(self._wcs, view))
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -367,6 +451,94 @@ class LazyMask(MaskBase):
 
     with_spectral_unit.__doc__ += with_spectral_unit_docs
 
+class LazyComparisonMask(LazyMask):
+
+    """
+    A boolean mask defined by the evaluation of a comparison function between a
+    fixed dataset and some other value.
+
+    This is conceptually similar to the :class:`LazyMask` but it will ensure
+    that the comparison value can be compared to the data
+
+    Parameters
+    ----------
+    function : callable
+        The function to apply to ``data``. This method should accept
+        a numpy array, which will be the data array passed to __init__,  and a
+        second argument also passed to __init__. It should return a boolean
+        array, where True values indicate that which pixels are
+        valid/unaffected by masking.
+    comparison_value : float or array
+        The comparison value for the array
+    data : array-like
+        The array to evaluate ``function`` on. This should support Numpy-like
+        slicing syntax.
+    wcs : `~astropy.wcs.WCS`
+        The WCS of the input data, which is used to define the coordinates
+        for which the boolean mask is defined.
+    """
+
+    def __init__(self, function, comparison_value, cube=None, data=None,
+                 wcs=None):
+        self._function = function
+        if cube is not None and (data is not None or wcs is not None):
+            raise ValueError("Pass only cube or (data & wcs)")
+        elif cube is not None:
+            self._data = cube._data
+            self._wcs = cube._wcs
+        elif data is not None and wcs is not None:
+            self._data = data
+            self._wcs = wcs
+        else:
+            raise ValueError("Either a cube or (data & wcs) is required.")
+
+        if (hasattr(comparison_value, 'shape') and not
+            is_broadcastable_and_smaller(self._data.shape,
+                                         comparison_value.shape)):
+            raise ValueError("The data and the comparison value cannot "
+                             "be broadcast to match shape")
+
+        self._comparison_value = comparison_value
+
+        self._wcs_whitelist = set()
+
+    def _include(self, data=None, wcs=None, view=()):
+        self._validate_wcs(data, wcs)
+
+        if hasattr(self._comparison_value, 'shape'):
+            cv_view = view_of_subset(self._comparison_value.shape,
+                                     self._data.shape, view)
+
+            return self._function(self._data[view],
+                                  self._comparison_value[cv_view])
+
+        else:
+            return self._function(self._data[view],
+                                  self._comparison_value)
+
+    def __getitem__(self, view):
+        if hasattr(self._comparison_value, 'shape'):
+            cv_view = view_of_subset(self._comparison_value.shape,
+                                     self._data.shape, view)
+            return LazyComparisonMask(self._function, data=self._data[view],
+                                      comparison_value=self._comparison_value[cv_view],
+                                      wcs=wcs_utils.slice_wcs(self._wcs, view))
+        else:
+            return LazyComparisonMask(self._function, data=self._data[view],
+                                      comparison_value=self._comparison_value,
+                                      wcs=wcs_utils.slice_wcs(self._wcs, view))
+
+    def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
+        """
+        Get a LazyComparisonMask copy with a WCS in the modified unit
+        """
+        newwcs = self._get_new_wcs(unit, velocity_convention, rest_value)
+
+        newmask = LazyComparisonMask(self._function, data=self._data,
+                                     comparison_value=self._comparison_value,
+                                     wcs=newwcs)
+        return newmask
+
 class FunctionMask(MaskBase):
 
     """
