@@ -1,5 +1,6 @@
 import pytest
 import itertools
+import operator
 import numpy as np
 from numpy.testing import assert_allclose
 from numpy.lib.stride_tricks import as_strided
@@ -7,9 +8,9 @@ from astropy.wcs import WCS
 from astropy import units as u
 
 from .test_spectral_cube import cube_and_raw
-from .. import (BooleanArrayMask, SpectralCube, LazyMask,
+from .. import (BooleanArrayMask, SpectralCube, LazyMask, LazyComparisonMask,
                 FunctionMask, CompositeMask)
-from ..masks import is_broadcastable
+from ..masks import is_broadcastable_and_smaller, dims_to_skip, view_of_subset
 
 from distutils.version import StrictVersion
 
@@ -67,6 +68,38 @@ def test_lazy_mask():
     assert_allclose(m._filled(data, wcs, view=(0, 0, slice(1, 4))), [np.nan, np.nan, 0])
     assert_allclose(m._flattened(data, wcs, view=(0, 0, slice(1, 4))), [0])
 
+def test_lazy_comparison_mask():
+
+    data = np.arange(5).reshape((1, 1, 5))
+    wcs = WCS()
+
+    m = LazyComparisonMask(operator.gt, 2, data=data, wcs=wcs)
+
+    assert_allclose(m.include(data, wcs), [[[0, 0, 0, 1, 1]]])
+    assert_allclose(m.exclude(data, wcs), [[[1, 1, 1, 0, 0]]])
+    assert_allclose(m._filled(data, wcs), [[[np.nan, np.nan, np.nan, 3, 4]]])
+    assert_allclose(m._flattened(data, wcs), [3, 4])
+
+    assert_allclose(m.include(data, wcs, view=(0, 0, slice(1, 4))), [0, 0, 1])
+    assert_allclose(m.exclude(data, wcs, view=(0, 0, slice(1, 4))), [1, 1, 0])
+    assert_allclose(m._filled(data, wcs, view=(0, 0, slice(1, 4))), [np.nan, np.nan, 3])
+    assert_allclose(m._flattened(data, wcs, view=(0, 0, slice(1, 4))), [3])
+
+    # Now if we call with different data, the results for include and exclude
+    # should *not* change.
+
+    data = (3 - np.arange(5)).reshape((1, 1, 5))
+
+    assert_allclose(m.include(data, wcs), [[[0, 0, 0, 1, 1]]])
+    assert_allclose(m.exclude(data, wcs), [[[1, 1, 1, 0, 0]]])
+    assert_allclose(m._filled(data, wcs), [[[np.nan, np.nan, np.nan, 0, -1]]])
+    assert_allclose(m._flattened(data, wcs), [0, -1])
+
+    assert_allclose(m.include(data, wcs, view=(0, 0, slice(1, 4))), [0, 0, 1])
+    assert_allclose(m.exclude(data, wcs, view=(0, 0, slice(1, 4))), [1, 1, 0])
+    assert_allclose(m._filled(data, wcs, view=(0, 0, slice(1, 4))), [np.nan, np.nan, 0])
+    assert_allclose(m._flattened(data, wcs, view=(0, 0, slice(1, 4))), [0])
+
 
 def test_function_mask_incorrect_shape():
 
@@ -256,6 +289,10 @@ def is_broadcastable_try(shp1, shp2):
     b = as_strided(x, shape=shp2, strides=[0] * len(shp2))
     try:
         c = np.broadcast_arrays(a, b)
+        # reverse order: compare last dim first (as broadcasting does)
+        if any(bi<ai for ai,bi in zip(shp1,shp2)):
+            # don't allow "b" to be smaller than "a"
+            return False
         return True
     except ValueError:
         return False
@@ -264,9 +301,23 @@ shapes = ([1,5,5], [1,5,1], [5,5,1], [5,5], [5,5,2],
           [2,3,4], [4,3,2], [4,2,3], [2,4,3])
 shape_combos = list(itertools.combinations(shapes,2))
 
-@pytest.mark.parametrize(('sh1','sh2'),shape_combos)
-def test_is_broadcastable(sh1, sh2):
-    assert is_broadcastable(sh1,sh2) == is_broadcastable_try(sh1,sh2)
+@pytest.mark.parametrize(('shp1','shp2'),shape_combos)
+def test_is_broadcastable(shp1, shp2):
+    assert is_broadcastable_and_smaller(shp1,shp2) == is_broadcastable_try(shp1,shp2)
+
+@pytest.mark.parametrize(('shp1','shp2','dim'),
+                         (([5,5],[2,5,5],[0]),
+                          #([5,5],[5,5,2],[2]),
+                          ([2,5,5],[2,5,5],[])))
+def test_dims_to_skip(shp1, shp2, dim):
+    assert dims_to_skip(shp1, shp2) == dim
+
+@pytest.mark.parametrize(('shp1','shp2', 'inview', 'outview'),
+                         (([5,5],[2,5,5],  [slice(0,1), slice(1,3), slice(2,4),], [slice(1,3), slice(2,4)]),
+                          # not a valid broadcast ([5,5],[5,5,2],  [slice(1,3), slice(2,4), slice(0,1),], [slice(1,3), slice(2,4)]),
+                          ([2,5,5],[2,5,5],[slice(0,1), slice(1,3), slice(2,4),], [slice(0,1), slice(1,3), slice(2,4),])))
+def test_view_of_subset(shp1, shp2, inview, outview):
+    assert view_of_subset(shp1,shp2,inview) == outview
 
 def test_flat_mask():
     cube, data = cube_and_raw('adv.fits')
@@ -295,4 +346,26 @@ def test_flat_mask_spectral():
     # Broadcast the 2D mask to 3D
     cubemask = (np.ones(4,dtype='bool')[:,None,None] & mask_array[None,:,:])
     # Check that spectral masking works too
-    assert np.all((data*cubemask).sum(axis=(1,2)) == mcube.sum(axis=(1,2)))
+    assert np.all((data*cubemask).sum(axis=(1,2)) ==
+                  mcube.sum(axis=(1,2)).value)
+
+def test_include():
+    cube, data = cube_and_raw('adv.fits')
+
+    mask_array = np.array([[True,False],[False,False],[True,True]])
+    bool_mask_array = BooleanArrayMask(mask=mask_array, wcs=cube._wcs,
+                                       shape=cube.shape)
+
+    assert np.all(bool_mask_array.include() == mask_array)
+
+def test_1d_mask():
+    # regression test for issue revealed in #183
+
+    cube, data = cube_and_raw('adv.fits')
+    mask = np.array([True, False, True, False])
+
+    sum0 = cube.with_mask(mask[:,None,None]).sum(axis=0)
+    sum0d = data[mask, :,:].sum(axis=0)
+
+    np.testing.assert_almost_equal(sum0.value, sum0d)
+
