@@ -1,6 +1,7 @@
 import pytest
 import numpy as np
 
+import astropy
 from astropy.wcs import WCS
 from astropy import units as u
 from astropy.io import fits
@@ -8,11 +9,13 @@ from astropy.io import fits
 from ..spectral_cube import SpectralCube
 from .helpers import assert_allclose
 
+from distutils.version import StrictVersion
+
 # the back of the book
 dv = 3e-2 * u.Unit('m/s')
 dy = 2e-5 * u.Unit('deg')
 dx = 1e-5 * u.Unit('deg')
-data_unit = u.dimensionless_unscaled
+data_unit = u.K
 
 m0v = np.array([[27, 30, 33],
                 [36, 39, 42],
@@ -45,6 +48,10 @@ m2x = np.array([[0.22222222, 0.63888889, 0.65759637],
                 [0.66222222, 0.66403682, 0.66493056],
                 [0.66543552, 0.66574839, 0.66595556]]) * dx ** 2
 MOMENTS = [[m0v, m0y, m0x], [m1v, m1y, m1x], [m2v, m2y, m2x]]
+# In issue 184, the cubes were corrected such that they all have valid units
+# Therefore, no separate tests are needed for moments-with-units and those
+# without
+MOMENTSu = MOMENTS
 
 
 def moment_cube():
@@ -65,6 +72,15 @@ axis_order = pytest.mark.parametrize(('axis', 'order'),
                                      (1, 0), (1, 1), (1, 2),
                                      (2, 0), (2, 1), (2, 2)))
 
+if StrictVersion(astropy.__version__[:3]) > StrictVersion('1.0'):
+    # The relative error is slightly larger on astropy-dev
+    # There is no obvious reason for this.
+    rtol = 2e-7
+    atol = 1e-40
+else:
+    rtol = 1e-7
+    atol = 0.0
+
 
 @axis_order
 def test_strategies_consistent(axis, order):
@@ -74,8 +90,8 @@ def test_strategies_consistent(axis, order):
     cwise = sc.moment(axis=axis, order=order, how='cube')
     swise = sc.moment(axis=axis, order=order, how='slice')
     rwise = sc.moment(axis=axis, order=order, how='ray')
-    assert_allclose(cwise, swise)
-    assert_allclose(cwise, rwise)
+    assert_allclose(cwise, swise, rtol=rtol, atol=atol)
+    assert_allclose(cwise, rwise, rtol=rtol, atol=atol)
 
 
 @pytest.mark.parametrize(('order', 'axis', 'how'),
@@ -94,13 +110,13 @@ def test_reference(order, axis, how):
 def test_consistent_mask_handling(axis, order):
     mc_hdu = moment_cube()
     sc = SpectralCube.read(mc_hdu)
-    sc._mask = sc > 4
+    sc._mask = sc > 4*u.K
 
     cwise = sc.moment(axis=axis, order=order, how='cube')
     swise = sc.moment(axis=axis, order=order, how='slice')
     rwise = sc.moment(axis=axis, order=order, how='ray')
-    assert_allclose(cwise, swise)
-    assert_allclose(cwise, rwise)
+    assert_allclose(cwise, swise, rtol=rtol, atol=atol)
+    assert_allclose(cwise, rwise, rtol=rtol, atol=atol)
 
 
 def test_convenience_methods():
@@ -118,5 +134,43 @@ def test_preserve_unit():
     m0 = sc_kms.moment0(axis=0)
     m1 = sc_kms.moment1(axis=0)
 
-    assert_allclose(m0, MOMENTS[0][0].to(u.km/u.s))
+    assert_allclose(m0, MOMENTS[0][0].to(u.K*u.km/u.s))
     assert_allclose(m1, MOMENTS[1][0].to(u.km/u.s))
+
+def test_with_flux_unit():
+    """
+    As of Issue 184, redundant with test_reference
+    """
+    mc_hdu = moment_cube()
+    sc = SpectralCube.read(mc_hdu)
+    sc._unit = u.K
+    sc_kms = sc.with_spectral_unit(u.km/u.s)
+    m0 = sc_kms.moment0(axis=0)
+    m1 = sc_kms.moment1(axis=0)
+
+    assert sc.unit == u.K
+    assert sc.filled_data[:].unit == u.K
+
+    assert_allclose(m0, MOMENTS[0][0].to(u.K*u.km/u.s))
+    assert_allclose(m1, MOMENTS[1][0].to(u.km/u.s))
+
+@pytest.mark.parametrize(('order', 'axis', 'how'),
+                         [(o, a, h)
+                          for o in [0, 1, 2]
+                          for a in [0, 1, 2]
+                          for h in ['cube', 'slice', 'auto', 'ray']])
+def test_how_withfluxunit(order, axis, how):
+    """
+    Regression test for issue 180
+    As of issue 184, this is mostly redundant with test_reference except that
+    it (kind of) checks that units are set
+    """
+    mc_hdu = moment_cube()
+    sc = SpectralCube.read(mc_hdu)
+    sc._unit = u.K
+    mom_sc = sc.moment(order=order, axis=axis, how=how)
+
+    assert sc.unit == u.K
+    assert sc.filled_data[:].unit == u.K
+
+    assert_allclose(mom_sc, MOMENTSu[order][axis])
