@@ -2,7 +2,7 @@ import numpy as np
 from astropy import units as u
 
 from .helpers import assert_allclose
-from ..spectral_cube import Projection
+from ..lower_dimensional_structures import Projection
 
 
 def test_copy_false():
