@@ -11,7 +11,8 @@ from .helpers import assert_allclose
 from . import path as data_path
 from ..spectral_axis import (convert_spectral_axis, determine_ctype_from_vconv,
                              cdelt_derivative, determine_vconv_from_ctype,
-                             get_rest_value_from_wcs)
+                             get_rest_value_from_wcs, air_to_vac,
+                             air_to_vac_deriv, vac_to_air)
 
 def test_cube_wcs_freqtovel():
     header = fits.Header.fromtextfile(data_path('cubewcs1.hdr'))
@@ -405,3 +406,114 @@ def test_change_rest_frequency(wcstype):
 
     assert_allclose(p_old, p_new, rtol=1e-3)
     assert_allclose(p_old, p_new, rtol=1e-3)
+
+
+# from http://classic.sdss.org/dr5/products/spectra/vacwavelength.html
+# these aren't accurate enough for my liking, but I can't find a better one readily
+air_vac = {
+    'H-beta':(4861.363, 4862.721)*u.AA,
+    '[O III]':(4958.911, 4960.295)*u.AA,
+    '[O III]':(5006.843, 5008.239)*u.AA,
+    '[N II]':(6548.05, 6549.86)*u.AA,
+    'H-alpha':(6562.801, 6564.614)*u.AA,
+    '[N II]':(6583.45, 6585.27)*u.AA,
+    '[S II]':(6716.44, 6718.29)*u.AA,
+    '[S II]':(6730.82, 6732.68)*u.AA,
+}
+           
+
+@pytest.mark.parametrize(('air','vac'), air_vac.values())
+def test_air_to_vac(air, vac):
+    # This is the accuracy provided by the line list we have.
+    # I'm not sure if the formula are incorrect or if the reference wavelengths
+    # are, but this is an accuracy of only 6 km/s, which is *very bad* for
+    # astrophysical applications.
+    assert np.abs((air_to_vac(air)- vac)) < 0.15*u.AA
+    assert np.abs((vac_to_air(vac)- air)) < 0.15*u.AA
+
+    assert np.abs((air_to_vac(air)- vac)/vac) < 2e-5
+    assert np.abs((vac_to_air(vac)- air)/air) < 2e-5
+
+    # round tripping
+    assert np.abs((vac_to_air(air_to_vac(air))-air))/air < 1e-8
+    assert np.abs((air_to_vac(vac_to_air(vac))-vac))/vac < 1e-8
+
+def test_byhand_awav2vel():
+    # AWAV
+    CRVAL3A = (6560*u.AA).to(u.m).value
+    CDELT3A = (1.0*u.AA).to(u.m).value
+    CUNIT3A = 'm'
+    CRPIX3A = 1.0
+    # restwav MUST be vacuum
+    restwl = air_to_vac(6562.81*u.AA)
+    RESTWAV = restwl.to(u.m).value
+    CRVAL3V = (CRVAL3A*u.m).to(u.m/u.s,
+                               u.doppler_optical(restwl)).value
+    CDELT3V = (CDELT3A*u.m*air_to_vac_deriv(CRVAL3A*u.m)/restwl) * constants.c
+    CUNIT3V = 'm/s'
+
+    mywcs = wcs.WCS(naxis=1)
+    mywcs.wcs.ctype[0] = 'AWAV'
+    mywcs.wcs.crval[0] = CRVAL3A
+    mywcs.wcs.crpix[0] = CRPIX3A
+    mywcs.wcs.cunit[0] = CUNIT3A
+    mywcs.wcs.cdelt[0] = CDELT3A
+    mywcs.wcs.restwav = RESTWAV
+    mywcs.wcs.set()
+
+
+    newwcs = convert_spectral_axis(mywcs, u.km/u.s,
+                                   determine_ctype_from_vconv(mywcs.wcs.ctype[0],
+                                                              u.km/u.s,
+                                                              'optical'))
+    
+    newwcs.wcs.set()
+    assert newwcs.wcs.cunit[0] == 'm / s'
+    np.testing.assert_almost_equal(newwcs.wcs.crval,
+                                   air_to_vac(CRVAL3A*u.m).to(u.m/u.s,
+                                                              u.doppler_optical(restwl)).value)
+    # Check that the cdelts match the expected cdelt, 1 angstrom / rest
+    # wavelength (vac)
+    np.testing.assert_almost_equal(newwcs.wcs.cdelt, CDELT3V.to(u.m/u.s).value)
+    # Check that the reference wavelength is 2.81 angstroms up
+    np.testing.assert_almost_equal(newwcs.wcs_pix2world((2.81,), 0), 0.0, decimal=3)
+
+
+    # Go through a full-on sanity check:
+    vline = 100*u.km/u.s
+    wave_line_vac = vline.to(u.AA, u.doppler_optical(restwl))
+    wave_line_air = vac_to_air(wave_line_vac)
+
+    pix_line_input = mywcs.wcs_world2pix((wave_line_air.to(u.m).value,), 0)
+    pix_line_output = newwcs.wcs_world2pix((vline.to(u.m/u.s).value,), 0)
+
+    np.testing.assert_almost_equal(pix_line_output, pix_line_input, decimal=4)
+
+def test_byhand_awav2wav():
+    # AWAV
+    CRVAL3A = (6560*u.AA).to(u.m).value
+    CDELT3A = (1.0*u.AA).to(u.m).value
+    CUNIT3A = 'm'
+    CRPIX3A = 1.0
+
+    mywcs = wcs.WCS(naxis=1)
+    mywcs.wcs.ctype[0] = 'AWAV'
+    mywcs.wcs.crval[0] = CRVAL3A
+    mywcs.wcs.crpix[0] = CRPIX3A
+    mywcs.wcs.cunit[0] = CUNIT3A
+    mywcs.wcs.cdelt[0] = CDELT3A
+    mywcs.wcs.set()
+
+
+    newwcs = convert_spectral_axis(mywcs, u.AA, 'WAVE')
+    newwcs.wcs.set()
+
+    np.testing.assert_almost_equal(newwcs.wcs_pix2world((0,),0),
+                                   air_to_vac(mywcs.wcs_pix2world((0,),0)*u.m).value)
+
+    np.testing.assert_almost_equal(newwcs.wcs_pix2world((10,),0),
+                                   air_to_vac(mywcs.wcs_pix2world((10,),0)*u.m).value)
+
+    # At least one of the components MUST change
+    assert not (mywcs.wcs.crval[0] == newwcs.wcs.crval[0]
+                and mywcs.wcs.crpix[0] == newwcs.wcs.crpix[0])
