@@ -65,16 +65,16 @@ def add_stokes_axis_to_wcs(wcs, add_before_ind):
     pc[inds[:, np.newaxis], inds[np.newaxis, :]] = wcs.wcs.get_pc()
     pc[add_before_ind, add_before_ind] = 1
 
-    def append_to_end(val, lst):
+    def append_to_posn(val, posn, lst):
         """ insert a value at index into a list """
-        return list(lst) + [val]
+        return list(lst)[:posn] + [val] + list(lst)[posn:]
 
-    outwcs.wcs.crpix = append_to_end(1, wcs.wcs.crpix)
-    outwcs.wcs.cdelt = append_to_end(1, wcs.wcs.get_cdelt())
-    outwcs.wcs.crval = append_to_end(1, wcs.wcs.crval)
-    outwcs.wcs.cunit = append_to_end("", wcs.wcs.cunit)
-    outwcs.wcs.ctype = append_to_end("STOKES", wcs.wcs.ctype)
-    outwcs.wcs.cname = append_to_end("STOKES", wcs.wcs.cname)
+    outwcs.wcs.crpix = append_to_posn(1, add_before_ind, wcs.wcs.crpix)
+    outwcs.wcs.cdelt = append_to_posn(1, add_before_ind, wcs.wcs.get_cdelt())
+    outwcs.wcs.crval = append_to_posn(1, add_before_ind, wcs.wcs.crval)
+    outwcs.wcs.cunit = append_to_posn("", add_before_ind, wcs.wcs.cunit)
+    outwcs.wcs.ctype = append_to_posn("STOKES", add_before_ind, wcs.wcs.ctype)
+    outwcs.wcs.cname = append_to_posn("STOKES", add_before_ind, wcs.wcs.cname)
     outwcs.wcs.pc = pc
 
     return outwcs
@@ -135,6 +135,20 @@ def reindex_wcs(wcs, inds):
     outwcs.wcs.cname = [wcs.wcs.cname[i] for i in inds]
     outwcs.wcs.pc = pc[inds[:, None], inds[None, :]]
 
+    pv_cards = []
+    for i, j in enumerate(inds):
+        for k, m, v in wcs.wcs.get_pv():
+            if k == j:
+                pv_cards.append((i, m, v))
+    outwcs.wcs.set_pv(pv_cards)
+
+    ps_cards = []
+    for i, j in enumerate(inds):
+        for k, m, v in wcs.wcs.get_ps():
+            if k == j:
+                ps_cards.append((i, m, v))
+    outwcs.wcs.set_ps(ps_cards)
+
     return outwcs
 
 
